#!/usr/bin/env python3

"""Module containing the haddock  class and the command line interface."""

import argparse
import glob
import os
import shutil
from typing import Optional

from biobb_common.configuration import settings
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.tools.file_utils import launchlogger


class Haddock3Accessibility(BiobbObject):
    """
    | biobb_haddock Haddock3Accessibility
    | Wrapper class for the Haddock-Restraints Accessibility module.
    | `Haddock-Restraints Accessibility <https://www.bonvinlab.org/haddock3/clients/haddock.clis.restraints.calc_accessibility.html>`_ computes residues accessibility using freesasa included in the Haddock3 package.

    Args:
        input_pdb_path (str): Path to the input PDB file. File type: input. `Sample file <https://raw.githubusercontent.com/bioexcel/biobb_haddock/master/biobb_haddock/test/data/haddock/e2aP_1F3G_noH.pdb>`_. Accepted formats: pdb (edam:format_1476).
        output_accessibility_path (str): Path to the output file with accessibility information. File type: output. `Sample file <https://raw.githubusercontent.com/bioexcel/biobb_haddock/master/biobb_haddock/test/reference/haddock_restraints/mol1_sasa.txt>`_. Accepted formats: txt (edam:format_2330), dat (edam:format_2330), out (edam:format_2330).
        output_actpass_path (str) (Optional): Path to the output file with active/passive residues to be used as haddock3 restraint information. File type: output. `Sample file <https://raw.githubusercontent.com/bioexcel/biobb_haddock/master/biobb_haddock/test/reference/haddock_restraints/mol1_haddock_actpass.txt>`_. Accepted formats: txt (edam:format_2330), dat (edam:format_2330), out (edam:format_2330).
        properties (dict - Python dictionary object containing the tool parameters, not input/output files):
            * **chain** (*str*) - ("A") Chain to be used from the input PDB file.
            * **cutoff** (*float*) - (0.4) Relative cutoff for sidechain accessibility.
            * **probe_radius** (*float*) - (1.4) Probe radius for the accessibility calculation.
            * **pass_to_act** (*bool*) - (False) If True, the passive residues become active in the actpass file and vice versa.
            * **binary_path** (*str*) - ("haddock") Path to the haddock haddock executable binary.
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.
            * **sandbox_path** (*str*) - ("./") [WF property] Parent path to the sandbox directory.
            * **container_path** (*str*) - (None)  Path to the binary executable of your container.
            * **container_image** (*str*) - (None) Container Image identifier.
            * **container_volume_path** (*str*) - ("/data") Path to an internal directory in the container.
            * **container_working_dir** (*str*) - (None) Path to the internal CWD in the container.
            * **container_user_id** (*str*) - (None) User number id to be mapped inside the container.
            * **container_shell_path** (*str*) - ("/bin/bash") Path to the binary executable of the container shell.


    Examples:
        This is a use example of how to use the building block from Python::

            from biobb_haddock.haddock_restraints.haddock3_accessibility import haddock3_accessibility
            prop = { 'cutoff': 0.4 }
            haddock3_accessibility(input_pdb_path='/path/to/mypdb.pdb',
                       output_accessibility_path='/path/to/output_report.txt',
                       properties=prop)

    Info:
        * wrapped_software:
            * name: Haddock33-restraints
            * version: 2025.5
            * license: Apache-2.0
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl
    """

    def __init__(
        self,
        input_pdb_path: str,
        output_accessibility_path: str,
        output_actpass_path: Optional[str] = None,
        properties: Optional[dict] = None,
        **kwargs,
    ) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)

        # Input/Output files
        self.io_dict = {
            "in": {
                "input_pdb_path": input_pdb_path,
            },
            "out": {
                "output_accessibility_path": output_accessibility_path,
                "output_actpass_path": output_actpass_path,
            },
        }

        # Properties specific for BB
        self.chain = properties.get("chain", "A")
        self.cutoff = properties.get("cutoff", 0.4)
        self.probe_radius = properties.get("probe_radius", 1.4)
        self.pass_to_act = properties.get("pass_to_act", False)

        # Properties specific for BB
        self.binary_path = properties.get("binary_path", "haddock3-restraints")

        # Check the properties
        self.check_properties(properties)

    @launchlogger
    def launch(self) -> int:
        """Execute the :class:`Haddock3Accessibility <biobb_haddock.haddock_restraints.haddock3_accessibility>` object."""

        # Setup Biobb
        if self.check_restart():
            return 0
        self.stage_files()

        # haddock3-restraints calc_accessibility 1UBQ.pdb --export_to_actpass
        self.cmd = [self.binary_path, "calc_accessibility", self.stage_io_dict['in']['input_pdb_path']]

        if self.io_dict["out"]["output_actpass_path"] is not None:
            self.cmd.append("--export_to_actpass")

        self.cmd.append(f"-c {self.cutoff}")
        self.cmd.append(">")
        self.cmd.append(self.stage_io_dict['out']['output_accessibility_path'])
        self.cmd.append("2>&1")

        # Run Biobb block
        self.run_biobb()

        # Check chain
        target_string = f"Chain {self.chain}"
        found = False
        with open(self.stage_io_dict['out']['output_accessibility_path'], 'r') as file:
            for line in file:
                if target_string in line:
                    found = True

        if found:
            # Rename/Copy output file to the given output file name
            file_name = os.path.basename(self.io_dict['in']['input_pdb_path'])
            shutil.copyfile(f"{file_name[:-4]}_passive_{self.chain}.actpass", self.io_dict["out"]["output_actpass_path"])
            if self.pass_to_act:
                with open(self.io_dict["out"]["output_actpass_path"], 'r') as file:
                    lines = file.readlines()
                with open(self.io_dict["out"]["output_actpass_path"], 'w') as file:
                    file.write(lines[1])
                    file.write('\n\n')
        else:
            print(f"\nWARNING: Chain {self.chain} not found in input PDB file. Please check and modify the chain property accordingly.\n")

        # Copy files to host
        self.copy_to_host()

        # Remove temporal files
        self.tmp_files.extend([self.stage_io_dict["unique_dir"]])
        actpass_files = glob.glob('*.actpass')
        self.tmp_files.extend(actpass_files)
        self.remove_tmp_files()

        return self.return_code


def haddock3_accessibility(
    input_pdb_path: str,
    output_accessibility_path: str,
    output_actpass_path: Optional[str] = None,
    properties: Optional[dict] = None,
    **kwargs,
) -> int:
    """Create :class:`Haddock3Accessibility <biobb_haddock.haddock_restraints.haddock3_accessibility>` class and
    execute the :meth:`launch() <biobb_haddock.haddock_restraints.haddock3_accessibility.launch>` method."""

    return Haddock3Accessibility(
        input_pdb_path=input_pdb_path,
        output_accessibility_path=output_accessibility_path,
        output_actpass_path=output_actpass_path,
        properties=properties,
        **kwargs,
    ).launch()


haddock3_accessibility.__doc__ = Haddock3Accessibility.__doc__


def main():
    parser = argparse.ArgumentParser(
        description="Wrapper of the haddock-restraints Accessibility module.",
        formatter_class=lambda prog: argparse.RawTextHelpFormatter(prog, width=99999),
    )
    parser.add_argument(
        "-c",
        "--config",
        required=False,
        help="This file can be a YAML file, JSON file or JSON string",
    )

    # Specific args of each building block
    required_args = parser.add_argument_group("required arguments")
    required_args.add_argument("--input_pdb_path", required=True)
    required_args.add_argument("--output_accessibility_path", required=True)
    parser.add_argument("--output_actpass_path", required=False)

    args = parser.parse_args()
    config = args.config if args.config else None
    properties = settings.ConfReader(config=config).get_prop_dic()

    # Specific call of each building block
    haddock3_accessibility(
        input_pdb_path=args.input_pdb_path,
        output_accessibility_path=args.output_accessibility_path,
        output_actpass_path=args.output_actpass_path,
        properties=properties,
    )


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

"""Module containing the Haddock3PassiveFromActive class and the command line interface."""

import argparse
from typing import Optional

from biobb_common.configuration import settings
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.tools.file_utils import launchlogger
from biobb_common.tools import file_utils as fu


class Haddock3PassiveFromActive(BiobbObject):
    """
    | biobb_haddock Haddock3PassiveFromActive
    | Wrapper class for the Haddock3-Restraints passive_from_active module.
    | `Haddock3-Restraints passive_from_active <https://www.bonvinlab.org/haddock3/clients/haddock.clis.restraints.passive_from_active.html>`_ given a list of active_residues and a PDB structure, it will return a list of surface exposed passive residues within a radius (6.5Å by default) from the active residues.

    Args:
        input_pdb_path (str): Path to the input PDB structure file. File type: input. `Sample file <https://raw.githubusercontent.com/bioexcel/biobb_haddock/master/biobb_haddock/test/data/haddock_restraints/1A2P_ch.pdb>`_. Accepted formats: pdb (edam:format_1476).
        output_actpass_path (str): Path to the output file with list of passive residues. File type: output. `Sample file <https://raw.githubusercontent.com/bioexcel/biobb_haddock/master/biobb_haddock/test/reference/haddock_restraints/1A2P_manual_actpass.txt>`_. Accepted formats: txt (edam:format_2330), dat (edam:format_2330), list (edam:format_2330), out (edam:format_2330).
        input_active_list_path (str) (Optional): Path to the input file with list of active residues. File type: input. Accepted formats: txt (edam:format_2330), dat (edam:format_2330), list (edam:format_2330).
        properties (dict - Python dictionary object containing the tool parameters, not input/output files):
            * **active_list** (*str*) - ('') List of active residues as a comma-separated string. Required if input_active_list_path is not provided.
            * **chain_id** (*str*) - (None) Chain ID to consider when calculating passive residues.
            * **surface_list_path** (*str*) - ("") Path to file with list of surface residues to filter.
            * **radius** (*float*) - (6.5) Radius in Angstroms to look for surface residues around active ones.
            * **binary_path** (*str*) - ("haddock3-restraints") Path to the haddock3-restraints executable binary.
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.
            * **sandbox_path** (*str*) - ("./") [WF property] Parent path to the sandbox directory.
            * **container_path** (*str*) - (None)  Path to the binary executable of your container.
            * **container_image** (*str*) - (None) Container Image identifier.
            * **container_volume_path** (*str*) - ("/data") Path to an internal directory in the container.
            * **container_working_dir** (*str*) - (None) Path to the internal CWD in the container.
            * **container_user_id** (*str*) - (None) User number id to be mapped inside the container.
            * **container_shell_path** (*str*) - ("/bin/bash") Path to the binary executable of the container shell.

    Examples:
        This is a use example of how to use the building block from Python::

            from biobb_haddock.haddock_restraints.haddock3_passive_from_active import haddock3_passive_from_active
            haddock3_passive_from_active(
                input_pdb_path='/path/to/structure.pdb',
                input_active_list_path='/path/to/active_residues.txt',
                output_actpass_path='/path/to/actpass.tbl.txt',
                properties={
                    'chain_id': 'A',
                    'radius': 6.5
                }
            )

    Info:
        * wrapped_software:
            * name: Haddock33-restraints
            * version: 2025.5
            * license: Apache-2.0
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl
    """

    def __init__(
        self,
        input_pdb_path: str,
        output_actpass_path: str,
        input_active_list_path: Optional[str] = None,
        properties: Optional[dict] = None,
        **kwargs,
    ) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)

        # Input/Output files
        self.io_dict = {
            "in": {
                "input_pdb_path": input_pdb_path
            },
            "out": {
                "output_actpass_path": output_actpass_path
            },
        }

        # Add input_active_list_path to io_dict if provided
        if input_active_list_path:
            self.io_dict["in"]["input_active_list_path"] = input_active_list_path

        # Properties specific for BB
        self.active_list = properties.get("active_list", "")
        self.chain_id = properties.get("chain_id", None)
        self.surface_list_path = properties.get("surface_list_path", "")
        self.radius = properties.get("radius", 6.5)
        self.binary_path = properties.get("binary_path", "haddock3-restraints")

        # Check that either input_active_list_path or active_list is provided
        if not input_active_list_path and not self.active_list:
            raise ValueError(
                "Either input_active_list_path or active_list property must be provided")

        # Check the properties
        self.check_properties(properties)

        # If surface_list_path is provided overwrite the active_list
        if self.surface_list_path:
            with open(self.surface_list_path, "r") as surface_file:
                self.active_list = surface_file.read()

    @launchlogger
    def launch(self) -> int:
        """Execute the :class:`Haddock3PassiveFromActive <biobb_haddock.haddock_restraints.haddock3_passive_from_active>` object."""

        # Setup Biobb
        if self.check_restart():
            return 0
        self.stage_files()

        # Build command line
        # haddock3-restraints passive_from_active <pdb_file> <active_list> [-c <chain_id>] [-s <surface_list>] [-r <radius>]
        self.cmd = [
            self.binary_path,
            "passive_from_active",
            self.stage_io_dict['in']['input_pdb_path'],
            self.active_list
        ]

        # Add optional parameters
        if self.chain_id:
            self.cmd.extend(["-c", self.chain_id])

        if self.surface_list_path:
            self.cmd.extend(
                ["-s", self.stage_io_dict['in']['surface_list_path']])

        # Radius not in this version
        # self.cmd.extend(["-r", str(self.radius)])

        # Redirect output to the output file
        self.cmd.append(">")
        self.cmd.append(self.stage_io_dict['out']['output_actpass_path'])
        self.cmd.append("2>&1")

        # Run Biobb block
        self.run_biobb()

        # Remove deprecation warning if present
        with open(self.stage_io_dict['out']['output_actpass_path'], 'r') as file:
            lines = file.readlines()
        fu.log('Result: ' + '\n'.join(lines), self.out_log, self.global_log)
        with open(self.stage_io_dict['out']['output_actpass_path'], 'w') as file:
            file.write(self.active_list.replace(",", " ")+"\n")
            if lines and "DEPRECATION NOTICE" in lines[0]:
                file.writelines(lines[1:])
            else:
                file.writelines(lines)

        # Copy files to host
        self.copy_to_host()

        # Remove temporal files
        self.tmp_files.extend([self.stage_io_dict["unique_dir"]])
        self.remove_tmp_files()

        return self.return_code


def haddock3_passive_from_active(
    input_pdb_path: str,
    output_actpass_path: str,
    input_active_list_path: Optional[str] = None,
    properties: Optional[dict] = None,
    **kwargs,
) -> int:
    """Create :class:`Haddock3PassiveFromActive <biobb_haddock.haddock_restraints.haddock3_passive_from_active>` class and
    execute the :meth:`launch() <biobb_haddock.haddock_restraints.haddock3_passive_from_active.launch>` method."""

    return Haddock3PassiveFromActive(
        input_pdb_path=input_pdb_path,
        output_actpass_path=output_actpass_path,
        input_active_list_path=input_active_list_path,
        properties=properties,
        **kwargs,
    ).launch()


haddock3_passive_from_active.__doc__ = Haddock3PassiveFromActive.__doc__


def main():
    parser = argparse.ArgumentParser(
        description="Wrapper of the haddock3-restraints passive_from_active module.",
        formatter_class=lambda prog: argparse.RawTextHelpFormatter(
            prog, width=99999),
    )
    parser.add_argument(
        "-c",
        "--config",
        required=False,
        help="This file can be a YAML file, JSON file or JSON string",
    )

    # Specific args of each building block
    required_args = parser.add_argument_group("required arguments")
    required_args.add_argument("--input_pdb_path", required=True)
    required_args.add_argument("--output_actpass_path", required=True)

    # Optional arguments
    parser.add_argument("--input_active_list_path", required=False)

    args = parser.parse_args()
    config = args.config if args.config else None
    properties = settings.ConfReader(config=config).get_prop_dic()

    # Specific call of each building block
    haddock3_passive_from_active(
        input_pdb_path=args.input_pdb_path,
        output_actpass_path=args.output_actpass_path,
        input_active_list_path=args.input_active_list_path,
        properties=properties,
    )


if __name__ == "__main__":
    main()



"""Common functions for package biobb_haddock.haddock"""

import logging
from typing import Any, Optional

import biobb_common.tools.file_utils as fu
from .haddock3_config import load, save

haddock_2_wf = {
    'ambig_fname': 'ambig_restraints_table_path',
    'unambig_fname': 'unambig_restraints_table_path',
    'hbond_fname': 'hb_restraints_table_path',
}


def create_cfg(
    output_cfg_path: str,
    workflow_dict: dict[str, Any],
    input_cfg_path: Optional[str] = None,
    cfg_properties_dict: Optional[dict[str, str]] = None,
    local_log: Optional[logging.Logger] = None,
    global_log: Optional[logging.Logger] = None,
) -> str:
    """Creates an CFG file using the following hierarchy  cfg_properties_dict > input_cfg_path > preset_dict"""
    cfg_dict: dict[str, Any] = {}

    # Handle input configuration if it exists
    if input_cfg_path:
        input_cfg = load(input_cfg_path)
        print(f"Input CFG: {input_cfg}")
        cfg_dict = input_cfg.copy()  # Start with entire loaded config as base

    # Apply single step configuration if specified
    if haddock_step_name := workflow_dict.get("haddock_step_name"):
        # Get preset properties for this step if any
        step_preset = cfg_preset(haddock_step_name)

        # Create or update the step configuration
        if not cfg_dict:
            # No input config, create new structure with single step
            target_key = haddock_step_name
            cfg_dict = {target_key: step_preset or {}}
        else:
            # Update the specific step in the existing config
            target_key = f"{haddock_step_name}.1"
            if target_key not in cfg_dict:
                cfg_dict[target_key] = {}
            # Merge preset values while preserving existing values
            if step_preset:
                for k, v in step_preset.items():
                    if k not in cfg_dict[target_key]:  # Only add if not already defined
                        cfg_dict[target_key][k] = v

        # Apply custom properties to the step
        if cfg_properties_dict:
            for k, v in cfg_properties_dict.items():
                fu.log(f"CFG: {k} = {v}", local_log, global_log)
                cfg_dict[target_key][k] = v
    else:
        # Multiple steps: haddock3_run and haddock3_extend
        if cfg_properties_dict:
            for key, value in cfg_properties_dict.items():
                if isinstance(value, dict):
                    # If the value is a dictionary, update the corresponding section in cfg_dict
                    if key not in cfg_dict:
                        cfg_dict[key] = {}
                    for sub_key, sub_value in value.items():
                        fu.log(f"CFG: {key}.{sub_key} = {sub_value}", local_log, global_log)
                        cfg_dict[key][sub_key] = sub_value
                else:
                    # If the value is not a dictionary, treat it as a top-level property
                    fu.log(f"CFG: {key} = {value}", local_log, global_log)
                    cfg_dict[key] = value
        # Add workflow_dict properties to cfg_dict
        for key, value in cfg_dict.items():
            if isinstance(value, dict):
                for sub_key, sub_value in value.items():
                    mapped_key = haddock_2_wf.get(sub_key)
                    if mapped_key and mapped_key in workflow_dict:
                        sub_value = workflow_dict[mapped_key]
                        cfg_dict[key][sub_key] = sub_value

    # Add molecules and run_dir if provided
    for key, value in workflow_dict.items():
        if key == 'haddock_step_name' or key in haddock_2_wf.values():
            continue
        cfg_dict[key] = value

    # Use haddock save
    save(cfg_dict, output_cfg_path)

    return output_cfg_path


def cfg_preset(haddock_step_name: str) -> dict[str, Any]:
    cfg_dict: dict[str, Any] = {}
    # cfg_dict["debug"] = True

    if haddock_step_name == "topoaa":
        cfg_dict["autohis"] = True
        cfg_dict["delenph"] = True
        cfg_dict["log_level"] = "quiet"
        cfg_dict["iniseed"] = 917
        cfg_dict["ligand_param_fname"] = ""
        cfg_dict["ligand_top_fname"] = ""
        cfg_dict["limit"] = True
        cfg_dict["tolerance"] = 0

    elif haddock_step_name == "rigidbody":
        cfg_dict["sampling"] = 20
        cfg_dict["tolerance"] = 20

    elif haddock_step_name == "seletop":
        cfg_dict["select"] = 5

    elif haddock_step_name == "flexref":
        cfg_dict["tolerance"] = 20

    elif haddock_step_name == "emref":
        cfg_dict["tolerance"] = 20

    return cfg_dict


def unzip_workflow_data(zip_file: str, out_log: Optional[logging.Logger] = None) -> str:
    """Extract all files in the zip_file and return the directory.

    Args:
        zip_file (str): Input topology zipball file path.
        out_log (:obj:`logging.Logger`): Input log object.

    Returns:
        str: Path to the extracted directory.

    """
    extract_dir = fu.create_unique_dir()
    zip_list = fu.unzip_list(zip_file, extract_dir, out_log)
    if out_log:
        out_log.info("Unzipping: ")
        out_log.info(zip_file)
        out_log.info("To: ")
        for file_name in zip_list:
            out_log.info(file_name)
    return extract_dir



#!/usr/bin/env python3

"""Module containing the haddock class and the command line interface."""

import argparse
from typing import Optional

from biobb_common.configuration import settings
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.tools.file_utils import launchlogger


class Haddock3RestrainBodies(BiobbObject):
    """
    | biobb_haddock Haddock3RestrainBodies
    | Wrapper class for the Haddock-Restraints restrain_bodies module.
    | `Haddock-Restraints restrain_bodies <https://www.bonvinlab.org/haddock3/clients/haddock.clis.restraints.restrain_bodies.html>`_ creates distance restraints to lock several chains together. Useful to avoid unnatural flexibility or movement due to sequence/numbering gaps.

    Args:
        input_structure_path (str): Path to the input PDB structure to be restrained. File type: input. `Sample file <https://raw.githubusercontent.com/bioexcel/biobb_haddock/master/biobb_haddock/test/data/haddock_restraints/4G6K_clean.pdb>`_. Accepted formats: pdb (edam:format_1476).
        output_tbl_path (str): Path to the output HADDOCK tbl file with Ambiguous Interaction Restraints (AIR) information. File type: output. `Sample file <https://raw.githubusercontent.com/bioexcel/biobb_haddock/master/biobb_haddock/test/reference/haddock_restraints/antibody-unambig.tbl>`_. Accepted formats: tbl (edam:format_2330), txt (edam:format_2330), out (edam:format_2330).
        properties (dict - Python dictionary object containing the tool parameters, not input/output files):
            * **exclude** (*str*) - (None) Chains to exclude from the calculation.
            * **verbose** (*int*) - (0) Tune verbosity of the output.
            * **binary_path** (*str*) - ("haddock3-restraints") Path to the HADDOCK3 restraints executable binary.
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.
            * **sandbox_path** (*str*) - ("./") [WF property] Parent path to the sandbox directory.
            * **container_path** (*str*) - (None)  Path to the binary executable of your container.
            * **container_image** (*str*) - (None) Container Image identifier.
            * **container_volume_path** (*str*) - ("/data") Path to an internal directory in the container.
            * **container_working_dir** (*str*) - (None) Path to the internal CWD in the container.
            * **container_user_id** (*str*) - (None) User number id to be mapped inside the container.
            * **container_shell_path** (*str*) - ("/bin/bash") Path to the binary executable of the container shell.


    Examples:
        This is a use example of how to use the building block from Python::

            from biobb_haddock.haddock_restraints.haddock3_restrain_bodies import haddock3_restrain_bodies
            haddock3_restrain_bodies(
                input_structure_path='/path/to/structure.pdb',
                output_tbl_path='/path/to/body_restraints.tbl'
            )

    Info:
        * wrapped_software:
            * name: Haddock33-restraints
            * version: 2025.5
            * license: Apache-2.0
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl
    """

    def __init__(
        self,
        input_structure_path: str,
        output_tbl_path: str,
        properties: Optional[dict] = None,
        **kwargs,
    ) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)

        # Input/Output files
        self.io_dict = {
            "in": {
                "input_structure_path": input_structure_path
            },
            "out": {
                "output_tbl_path": output_tbl_path,
            },
        }

        # Properties specific for BB
        self.binary_path = properties.get("binary_path", "haddock3-restraints")
        self.exclude = properties.get("exclude", None)
        self.verbose = properties.get("verbose", 0)

        # Check the properties
        self.check_properties(properties)

    @launchlogger
    def launch(self) -> int:
        """Execute the :class:`Haddock3RestrainBodies <biobb_haddock.haddock_restraints.haddock3_restrain_bodies>` object."""

        # Setup Biobb
        if self.check_restart():
            return 0
        self.stage_files()

        # haddock3-restraints restrain_bodies <structure> [--exclude] [--verbose]
        self.cmd = [self.binary_path, "restrain_bodies",
                    self.stage_io_dict['in']['input_structure_path']]

        if self.exclude is not None:
            self.cmd.extend(["--exclude", self.exclude])

        if self.verbose > 0:
            self.cmd.extend(["--verbose", str(self.verbose)])

        self.cmd.append(">")
        self.cmd.append(self.stage_io_dict['out']['output_tbl_path'])
        self.cmd.append("2>&1")

        # Run Biobb block
        self.run_biobb()

        # Remove deprecation warning if present
        with open(self.stage_io_dict['out']['output_tbl_path'], 'r') as file:
            lines = file.readlines()
        if lines and "DEPRECATION NOTICE" in lines[0]:
            with open(self.stage_io_dict['out']['output_tbl_path'], 'w') as file:
                file.writelines(lines[1:])

        # Copy files to host
        self.copy_to_host()

        # Remove temporal files
        self.tmp_files.extend([self.stage_io_dict["unique_dir"]])
        self.remove_tmp_files()

        return self.return_code


def haddock3_restrain_bodies(
    input_structure_path: str,
    output_tbl_path: str,
    properties: Optional[dict] = None,
    **kwargs,
) -> int:
    """Create :class:`Haddock3RestrainBodies <biobb_haddock.haddock_restraints.haddock3_restrain_bodies>` class and
    execute the :meth:`launch() <biobb_haddock.haddock_restraints.haddock3_restrain_bodies.launch>` method."""

    return Haddock3RestrainBodies(
        input_structure_path=input_structure_path,
        output_tbl_path=output_tbl_path,
        properties=properties,
        **kwargs,
    ).launch()


haddock3_restrain_bodies.__doc__ = Haddock3RestrainBodies.__doc__


def main():
    parser = argparse.ArgumentParser(
        description="Wrapper of the haddock-restraints restrain_bodies module.",
        formatter_class=lambda prog: argparse.RawTextHelpFormatter(
            prog, width=99999),
    )
    parser.add_argument(
        "-c",
        "--config",
        required=False,
        help="This file can be a YAML file, JSON file or JSON string",
    )

    # Specific args of each building block
    required_args = parser.add_argument_group("required arguments")
    required_args.add_argument("--input_structure_path", required=True)
    required_args.add_argument("--output_tbl_path", required=True)

    args = parser.parse_args()
    config = args.config if args.config else None
    properties = settings.ConfReader(config=config).get_prop_dic()

    # Specific call of each building block
    haddock3_restrain_bodies(
        input_structure_path=args.input_structure_path,
        output_tbl_path=args.output_tbl_path,
        properties=properties,
    )


if __name__ == "__main__":
    main()



"""
Disclaimer
This code was obtained from the HADDOCK3 repository:
https://github.com/haddocking/haddock3/blob/main/src/haddock/gear/config.py
"""
import toml
import os
import re
from pathlib import Path

# the re.ASCII parameter makes sure non-ascii chars are rejected in the \w key

# Captures the main headers.
# https://regex101.com/r/9urqti/1
_main_header_re = re.compile(r"^ *\[(\w+)\]", re.ASCII)

# regex by @sverhoeven
# Matches ['<name>.<digit>']
_main_quoted_header_re = re.compile(r"^ *\[\'(\w+)\.\d+\'\]", re.ASCII)

# Captures sub-headers
# https://regex101.com/r/6OpJJ8/1
# thanks https://stackoverflow.com/questions/39158902
_sub_header_re = re.compile(r"^ *\[(\w+)((?:\.\w+)+)\]", re.ASCII)

# regex by @sverhoeven
_sub_quoted_header_re = re.compile(
    r"^ *\[\'(\w+)\.\d+\'((?:\.\w+)+)\]",
    re.ASCII,
)

# Captures parameter uppercase boolean
_uppercase_bool_re = re.compile(r"(_?\w+((_?\w+?)+)?\s*=\s*)(True|False)", re.ASCII)


def load(fpath: str) -> dict[str, dict[str, dict[str, str]]]:
    """
    Load an HADDOCK3 configuration file to a dictionary.

    Accepts HADDOCK3 ``cfg`` files or pure ``toml`` files.

    Parameters
    ----------
    fpath : str or :external:py:class:`pathlib.Path`
        Path to user configuration file.

    Returns
    -------
    dictionary
        Representing the user configuration file where first level of
        keys are the module names. Step keys will have a numeric
        suffix, for example: ``module.1``.

    .. see-also::
        * :py:func:`loads`
    """
    try:
        return loads(Path(fpath).read_text())
    except Exception as err:
        raise Exception(
            "Something is wrong with the config file."
        ) from err  # noqa: E501


def loads(cfg_str: str) -> dict[str, dict[str, dict[str, str]]]:
    """
    Read a string representing a config file to a dictionary.

    Config strings are converted to toml-compatible format and finally
    read by the toml library.

    All headers (dictionary keys) will be suffixed by an integer
    starting at ``1``. For example: ``topoaa.1``. If the key is
    repeated, ``2`` is appended, and so forth. Even if specific
    integers are provided by the user, the suffix integers will be
    normalized.

    Parameters
    ----------
    cfg_str : str
        The string representing the config file. Accepted formats are
        the HADDOCK3 config file or pure `toml` syntax.

    Returns
    -------
    all_configs : dict
        A dictionary holding all the configuration file steps:

        - 'raw_input': Original input file as provided by user.
        - 'cleaned_input': Regex cleaned input file.
        - 'loaded_cleaned_input': Dict of toml loaded cleaned input.
        - 'final_cfg': The config in the form of a dictionary. In which
          the order of the keys matters as it defines the order of the
          steps in the workflow.
    """
    new_lines: list[str] = []
    cfg_lines = cfg_str.split(os.linesep)
    counter: dict[str, int] = {}

    # this for-loop normalizes all headers regardless of their input format.
    for line in cfg_lines:
        if group := _main_header_re.match(line):
            name = group[1]
            counter.setdefault(name, 0)
            counter[name] += 1
            count = counter[name]
            new_line = f"['{name}.{count}']"

        elif group := _main_quoted_header_re.match(line):
            name = group[1]
            counter.setdefault(name, 0)
            counter[name] += 1
            count = counter[name]
            new_line = f"['{name}.{count}']"

        elif group := _sub_header_re.match(line):
            name = group[1]
            count = counter[name]  # name should be already defined here
            new_line = f"['{name}.{count}'{group[2]}]"

        elif group := _sub_quoted_header_re.match(line):
            name = group[1]
            count = counter[name]  # name should be already defined here
            new_line = f"['{name}.{count}'{group[2]}]"

        elif group := _uppercase_bool_re.match(line):
            param = group[1]  # Catches 'param = '
            uppercase_bool = group[4]
            new_line = f"{param}{uppercase_bool.lower()}"  # Lowercase bool

        else:
            new_line = line

        new_lines.append(new_line)

    # Re-build workflow configuration file
    cfg = os.linesep.join(new_lines)

    try:
        cfg_dict = toml.loads(cfg)  # Try to load it with the toml library
    except Exception as err:
        print(cfg)
        raise Exception(
            "Some thing is wrong with the config file: " f"{str(err)}"
        ) from err

    return cfg_dict


def save(cfg_dict: dict, path: str) -> None:
    """
    Write a dictionary to a HADDOCK3 config file.

    Write the HADDOCK3 parameter dictionary to a `.cfg` file. There is
    also the option to write in pure TOML format. Both are compatible with
    HADDOCK3.

    Parameters
    ----------
    cfg_dict : dict
        The dictionary containing the parameters.

    path : str or pathlib.Path
        File name where to save the configuration file.

    """

    with open(path, "w") as fout:
        toml.dump(cfg_dict, fout)



name = "haddock"
__all__ = [
    "capri_eval",
    "clust_fcc",
    "contact_map",
    "em_ref",
    "flex_ref",
    "haddock3_extend",
    "haddock3_run",
    "rigid_body",
    "sele_top_clusts",
    "sele_top",
    "topology",
]



#!/usr/bin/env python3

"""Module containing the haddock FlexRef class and the command line interface."""

import argparse
import shutil
from pathlib import Path
from typing import Optional

from biobb_common.configuration import settings
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.tools import file_utils as fu
from biobb_common.tools.file_utils import launchlogger

from biobb_haddock.haddock.common import create_cfg, unzip_workflow_data


class FlexRef(BiobbObject):
    """
    | biobb_haddock FlexRef
    | Wrapper class for the Haddock FlexRef module.
    | The FlexRef module.`Haddock FlexRef module <https://www.bonvinlab.org/haddock3/modules/refinement/haddock.modules.refinement.flexref.html>`_ computes a flexible refinement over selected structures.

    Args:
        input_haddock_wf_data_zip (str): Path to the input zipball containing all the current Haddock workflow data. File type: input. `Sample file <https://github.com/bioexcel/biobb_haddock/raw/master/biobb_haddock/test/data/haddock/haddock_wf_data_topology.zip>`_. Accepted formats: zip (edam:format_3987).
        refinement_output_zip_path (str): Path to the output PDB file collection in zip format. File type: output. `Sample file <https://raw.githubusercontent.com/bioexcel/biobb_haddock/master/biobb_haddock/test/reference/haddock/ref_rigidbody.zip>`_. Accepted formats: zip (edam:format_3987).
        ambig_restraints_table_path (str) (Optional): Path to the input TBL file containing a list of ambiguous restraints for docking. File type: input. `Sample file <https://raw.githubusercontent.com/bioexcel/biobb_haddock/master/biobb_haddock/test/data/haddock/e2a-hpr_air.tbl>`_. Accepted formats: tbl (edam:format_2330).
        unambig_restraints_table_path (str) (Optional): Path to the input TBL file containing a list of unambiguous restraints for docking. File type: input. `Sample file <https://raw.githubusercontent.com/bioexcel/biobb_haddock/master/biobb_haddock/test/data/haddock/e2a-hpr_air.tbl>`_. Accepted formats: tbl (edam:format_2330).
        hb_restraints_table_path (str) (Optional): Path to the input TBL file containing a list of hydrogen bond restraints for docking. File type: input. `Sample file <https://raw.githubusercontent.com/bioexcel/biobb_haddock/master/biobb_haddock/test/data/haddock/e2a-hpr_air.tbl>`_. Accepted formats: tbl (edam:format_2330).
        output_haddock_wf_data_zip (str) (Optional): Path to the output zipball containing all the current Haddock workflow data. File type: output. `Sample file <https://github.com/bioexcel/biobb_haddock/raw/master/biobb_haddock/test/data/haddock/haddock_wf_data_emref.zip>`_. Accepted formats: zip (edam:format_3987).
        haddock_config_path (str) (Optional): Haddock configuration CFG file path. File type: input. `Sample file <https://raw.githubusercontent.com/bioexcel/biobb_haddock/master/biobb_haddock/test/data/haddock/run.cfg>`_. Accepted formats: cfg (edam:format_1476).
        properties (dict - Python dictionary object containing the tool parameters, not input/output files):
            * **cfg** (*dict*) - ({}) Haddock configuration options specification.
            * **global_cfg** (*dict*) - ({"postprocess": False}) `Global configuration options <https://www.bonvinlab.org/haddock3-user-manual/global_parameters.html>`_ specification.
            * **binary_path** (*str*) - ("haddock") Path to the haddock haddock executable binary.
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.
            * **sandbox_path** (*str*) - ("./") [WF property] Parent path to the sandbox directory.
            * **container_path** (*str*) - (None)  Path to the binary executable of your container.
            * **container_image** (*str*) - (None) Container Image identifier.
            * **container_volume_path** (*str*) - ("/data") Path to an internal directory in the container.
            * **container_working_dir** (*str*) - (None) Path to the internal CWD in the container.
            * **container_user_id** (*str*) - (None) User number id to be mapped inside the container.
            * **container_shell_path** (*str*) - ("/bin/bash") Path to the binary executable of the container shell.


    Examples:
        This is a use example of how to use the building block from Python::

            from biobb_haddock.haddock.flex_ref import flex_ref
            prop = { 'binary_path': 'haddock' }
            flex_ref(input_haddock_wf_data_zip='/path/to/myworkflowdata.zip',
                     refinement_output_zip_path='/path/to/mydockingstructures.zip',
                     properties=prop)

    Info:
        * wrapped_software:
            * name: Haddock3
            * version: 2025.5
            * license: Apache-2.0
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl
    """

    def __init__(
        self,
        input_haddock_wf_data_zip: str,
        refinement_output_zip_path: str,
        ambig_restraints_table_path: Optional[str] = None,
        unambig_restraints_table_path: Optional[str] = None,
        hb_restraints_table_path: Optional[str] = None,
        output_haddock_wf_data_zip: Optional[str] = None,
        haddock_config_path: Optional[str] = None,
        properties: Optional[dict] = None,
        **kwargs,
    ) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)

        # Input/Output files
        self.io_dict = {
            "in": {
                "ambig_restraints_table_path": ambig_restraints_table_path,
                "unambig_restraints_table_path": unambig_restraints_table_path,
                "hb_restraints_table_path": hb_restraints_table_path,
                "haddock_config_path": haddock_config_path,
            },
            "out": {
                "output_haddock_wf_data_zip": output_haddock_wf_data_zip,
                "refinement_output_zip_path": refinement_output_zip_path,
            },
        }
        # Should not be copied inside container
        self.input_haddock_wf_data_zip = input_haddock_wf_data_zip

        # Properties specific for BB
        self.haddock_step_name = "flexref"
        self.output_cfg_path = properties.get("output_cfg_path", "haddock.cfg")
        self.cfg = {k: v for k, v in properties.get("cfg", dict()).items()}
        self.global_cfg = properties.get("global_cfg", dict(postprocess=False))

        # Properties specific for BB
        self.binary_path = properties.get("binary_path", "haddock3")

        # Check the properties
        self.check_properties(properties)

    @launchlogger
    def launch(self) -> int:
        """Execute the :class:`FlexRef <biobb_haddock.haddock.flex_ref>` object."""
        # tmp_files = []

        # Setup Biobb
        if self.check_restart():
            return 0
        self.stage_files()

        # Unzip workflow data to workflow_data_out
        run_dir = unzip_workflow_data(
            zip_file=self.input_haddock_wf_data_zip, out_log=self.out_log
        )

        workflow_dict = {"haddock_step_name": self.haddock_step_name}
        workflow_dict.update(self.global_cfg)

        if ambig_path := self.stage_io_dict["in"].get("ambig_restraints_table_path"):
            self.cfg["ambig_fname"] = ambig_path

        if unambig_fname := self.stage_io_dict["in"].get("unambig_restraints_table_path"):
            self.cfg["unambig_fname"] = unambig_fname

        if hbond_fname := self.stage_io_dict["in"].get("hb_restraints_table_path"):
            self.cfg["hbond_fname"] = hbond_fname

        # Create data dir
        cfg_dir = fu.create_unique_dir()
        self.output_cfg_path = create_cfg(
            output_cfg_path=str(Path(cfg_dir).joinpath(self.output_cfg_path)),
            workflow_dict=workflow_dict,
            input_cfg_path=self.stage_io_dict["in"].get("haddock_config_path"),
            cfg_properties_dict=self.cfg,
            local_log=self.out_log,
            global_log=self.global_log,
        )

        if self.container_path:
            fu.log("Container execution enabled", self.out_log)

            shutil.copy2(self.output_cfg_path, self.stage_io_dict.get("unique_dir", ""))
            self.output_cfg_path = str(
                Path(self.container_volume_path).joinpath(
                    Path(self.output_cfg_path).name
                )
            )

            shutil.copytree(
                run_dir,
                str(
                    Path(self.stage_io_dict.get("unique_dir", "")).joinpath(
                        Path(run_dir).name
                    )
                ),
            )
            run_dir = str(
                Path(self.stage_io_dict.get("unique_dir", "")).joinpath(
                    Path(run_dir).name
                )
            )

        self.cmd = [self.binary_path, self.output_cfg_path, "--extend-run", run_dir]

        # Run Biobb block
        self.run_biobb()

        # Copy files to host
        # self.copy_to_host()

        # Copy output

        haddock_output_list = [
            str(path)
            for path in Path(run_dir).iterdir()
            if path.is_dir() and str(path).endswith(workflow_dict["haddock_step_name"])
        ]
        haddock_output_list.sort(reverse=True)
        output_file_list = list(
            Path(haddock_output_list[0]).glob(
                workflow_dict["haddock_step_name"] + r"*.pdb*"
            )
        )
        fu.zip_list(
            self.io_dict["out"]["refinement_output_zip_path"],
            output_file_list,
            self.out_log,
        )

        # Create zip output
        if self.io_dict["out"].get("output_haddock_wf_data_zip"):
            fu.log(
                f"Zipping {run_dir} to {str(Path(self.io_dict['out']['output_haddock_wf_data_zip']).with_suffix(''))} ",
                self.out_log,
                self.global_log,
            )
            shutil.make_archive(
                str(
                    Path(self.io_dict["out"]["output_haddock_wf_data_zip"]).with_suffix(
                        ""
                    )
                ),
                "zip",
                run_dir,
            )

        # Remove temporal files
        self.tmp_files.extend([run_dir,
                               cfg_dir,
                               self.stage_io_dict.get("unique_dir")
                               ])
        self.remove_tmp_files()

        return self.return_code


def flex_ref(
    input_haddock_wf_data_zip: str,
    refinement_output_zip_path: str,
    ambig_restraints_table_path: Optional[str] = None,
    unambig_restraints_table_path: Optional[str] = None,
    hb_restraints_table_path: Optional[str] = None,
    output_haddock_wf_data_zip: Optional[str] = None,
    haddock_config_path: Optional[str] = None,
    properties: Optional[dict] = None,
    **kwargs,
) -> int:
    """Create :class:`FlexRef <biobb_haddock.haddock.flex_ref>` class and
    execute the :meth:`launch() <biobb_haddock.haddock.flex_ref.launch>` method."""

    return FlexRef(
        input_haddock_wf_data_zip=input_haddock_wf_data_zip,
        refinement_output_zip_path=refinement_output_zip_path,
        ambig_restraints_table_path=ambig_restraints_table_path,
        unambig_restraints_table_path=unambig_restraints_table_path,
        hb_restraints_table_path=hb_restraints_table_path,
        output_haddock_wf_data_zip=output_haddock_wf_data_zip,
        haddock_config_path=haddock_config_path,
        properties=properties,
        **kwargs,
    ).launch()


def main():
    parser = argparse.ArgumentParser(
        description="Wrapper of the haddock FlexRef module.",
        formatter_class=lambda prog: argparse.RawTextHelpFormatter(prog, width=99999),
    )
    parser.add_argument(
        "-c",
        "--config",
        required=False,
        help="This file can be a YAML file, JSON file or JSON string",
    )

    # Specific args of each building block
    required_args = parser.add_argument_group("required arguments")
    required_args.add_argument("--input_haddock_wf_data_zip", required=True)
    required_args.add_argument("--refinement_output_zip_path", required=True)
    parser.add_argument("--ambig_restraints_table_path", required=False)
    parser.add_argument("--unambig_restraints_table_path", required=False)
    parser.add_argument("--hb_restraints_table_path", required=False)
    parser.add_argument("--output_haddock_wf_data_zip", required=False)
    parser.add_argument("--haddock_config_path", required=False)

    args = parser.parse_args()
    config = args.config if args.config else None
    properties = settings.ConfReader(config=config).get_prop_dic()

    # Specific call of each building block
    flex_ref(
        input_haddock_wf_data_zip=args.input_haddock_wf_data_zip,
        refinement_output_zip_path=args.refinement_output_zip_path,
        ambig_restraints_table_path=args.ambig_restraints_table_path,
        unambig_restraints_table_path=args.unambig_restraints_table_path,
        hb_restraints_table_path=args.hb_restraints_table_path,
        output_haddock_wf_data_zip=args.output_haddock_wf_data_zip,
        haddock_config_path=args.haddock_config_path,
        properties=properties,
    )


if __name__ == "__main__":
    main()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="biobb_haddock",
    version="5.1.0",
    author="Biobb developers",
    author_email="ruben.chaves@irbbarcelona.org",
    description="biobb_haddock is the Biobb module collection to compute information-driven flexible protein-protein docking.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords="Bioinformatics Workflows BioExcel Compatibility",
    url="https://github.com/bioexcel/biobb_haddock",
    project_urls={
        "Documentation": "http://biobb_haddock.readthedocs.io/en/latest/",
        "Bioexcel": "https://bioexcel.eu/",
    },
    packages=setuptools.find_packages(exclude=["docs", "test"]),
    package_data={"biobb_haddock": ["py.typed"]},
    include_package_data=True,
    install_requires=["biobb_common==5.1.0"],
    python_requires=">=3.9",
    entry_points={
        "console_scripts": [
            "capri_eval = biobb_haddock.haddock.capri_eval:main",
            "clust_fcc = biobb_haddock.haddock.clust_fcc:main",
            "contact_map = biobb_haddock.haddock.contact_map:main",
            "em_ref = biobb_haddock.haddock.em_ref:main",
            "flex_ref = biobb_haddock.haddock.flex_ref:main",
            "haddock3_extend = biobb_haddock.haddock.haddock3_extend:main",
            "haddock3_run = biobb_haddock.haddock.haddock3_run:main",
            "rigid_body = biobb_haddock.haddock.rigid_body:main",
            "sele_top_clusts = biobb_haddock.haddock.sele_top_clusts:main",
            "sele_top = biobb_haddock.haddock.sele_top:main",
            "topology = biobb_haddock.haddock.topology:main",
            "haddock3_accessibility = biobb_haddock.haddock_restraints.haddock3_accessibility:main",
            "haddock3_actpass_to_ambig = biobb_haddock.haddock_restraints.haddock3_actpass_to_ambig:main",
            "haddock3_passive_from_active = biobb_haddock.haddock_restraints.haddock3_passive_from_active:main",
            "haddock3_restrain_bodies = biobb_haddock.haddock_restraints.haddock3_restrain_bodies:main",
        ]
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX",
        "Operating System :: Unix",
    ],
)



#!/usr/bin/env python3

"""Module containing the haddock Topology class and the command line interface."""

# import os
# import json
import argparse
import shutil
from pathlib import Path
from typing import Optional

from biobb_common.configuration import settings
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.tools import file_utils as fu
from biobb_common.tools.file_utils import launchlogger

from biobb_haddock.haddock.common import create_cfg


class Topology(BiobbObject):
    """
    | biobb_haddock Topology
    | Wrapper class for the Haddock Topology module.
    | The Topology module. `Haddock Topology module <https://www.bonvinlab.org/haddock3/modules/topology/haddock.modules.topology.topoaa.html#haddock.modules.topology.topoaa.HaddockModule>`_ creates a topology from a system to be used for docking.

    Args:
        mol1_input_pdb_path (str): Path to the input PDB file. File type: input. `Sample file <https://raw.githubusercontent.com/bioexcel/biobb_haddock/master/biobb_haddock/test/data/haddock/e2aP_1F3G.pdb>`_. Accepted formats: pdb (edam:format_1476).
        mol1_output_top_zip_path (str): Path to the output PDB file collection in zip format. File type: output. `Sample file <https://raw.githubusercontent.com/bioexcel/biobb_haddock/master/biobb_haddock/test/reference/haddock/ref_mol1_top.zip>`_. Accepted formats: zip (edam:format_3987).
        mol2_input_pdb_path (str) (Optional): Path to the input PDB file. File type: input. `Sample file <https://raw.githubusercontent.com/bioexcel/biobb_haddock/master/biobb_haddock/test/data/haddock/hpr_ensemble.pdb>`_. Accepted formats: pdb (edam:format_1476).
        mol2_output_top_zip_path (str) (Optional): Path to the output PDB file collection in zip format. File type: output. `Sample file <https://raw.githubusercontent.com/bioexcel/biobb_haddock/master/biobb_haddock/test/reference/haddock/ref_mol2_top.zip>`_. Accepted formats: zip (edam:format_3987).
        output_haddock_wf_data_zip (str) (Optional): Path to the output zipball containing all the current Haddock workflow data. File type: output. `Sample file <https://github.com/bioexcel/biobb_haddock/raw/master/biobb_haddock/test/data/haddock/haddock_wf_data_emref.zip>`_. Accepted formats: zip (edam:format_3987).
        haddock_config_path (str) (Optional): Haddock configuration CFG file path. File type: input. `Sample file <https://raw.githubusercontent.com/bioexcel/biobb_haddock/master/biobb_haddock/test/data/haddock/run.cfg>`_. Accepted formats: cfg (edam:format_1476).
        properties (dict - Python dictionary object containing the tool parameters, not input/output files):
            * **cfg** (*dict*) - ({}) Haddock configuration options specification.
            * **global_cfg** (*dict*) - ({"postprocess": False}) `Global configuration options <https://www.bonvinlab.org/haddock3-user-manual/global_parameters.html>`_ specification.
            * **binary_path** (*str*) - ("haddock") Path to the haddock haddock executable binary.
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.
            * **sandbox_path** (*str*) - ("./") [WF property] Parent path to the sandbox directory.
            * **container_path** (*str*) - (None)  Path to the binary executable of your container.
            * **container_image** (*str*) - (None) Container Image identifier.
            * **container_volume_path** (*str*) - ("/data") Path to an internal directory in the container.
            * **container_working_dir** (*str*) - (None) Path to the internal CWD in the container.
            * **container_user_id** (*str*) - (None) User number id to be mapped inside the container.
            * **container_shell_path** (*str*) - ("/bin/bash") Path to the binary executable of the container shell.


    Examples:
        This is a use example of how to use the building block from Python::

            from biobb_haddock.haddock.topology import topology
            prop = { 'binary_path': 'haddock' }
            topology(mol1_input_pdb_path='/path/to/myStructure.pdb',
                     mol1_output_top_zip_path='/path/to/topology.zip',
                     properties=prop)

    Info:
        * wrapped_software:
            * name: Haddock3
            * version: 2025.5
            * license: Apache-2.0
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl
    """

    def __init__(
        self,
        mol1_input_pdb_path: str,
        mol1_output_top_zip_path: str,
        mol2_input_pdb_path: Optional[str] = None,
        mol2_output_top_zip_path: Optional[str] = None,
        output_haddock_wf_data_zip: Optional[str] = None,
        haddock_config_path: Optional[str] = None,
        properties: Optional[dict] = None,
        **kwargs,
    ) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)

        # Input/Output files
        self.io_dict = {
            "in": {
                "mol1_input_pdb_path": mol1_input_pdb_path,
                "mol2_input_pdb_path": mol2_input_pdb_path,
                "haddock_config_path": haddock_config_path,
            },
            "out": {
                "output_haddock_wf_data_zip": output_haddock_wf_data_zip,
                "mol1_output_top_zip_path": mol1_output_top_zip_path,
                "mol2_output_top_zip_path": mol2_output_top_zip_path,
            },
        }

        # Properties specific for BB
        self.haddock_step_name = "topoaa"
        self.output_cfg_path = properties.get("output_cfg_path", "haddock.cfg")
        self.cfg = {k: v for k, v in properties.get("cfg", dict()).items()}
        self.global_cfg = properties.get("global_cfg", dict(postprocess=False))

        # Properties specific for BB
        self.binary_path = properties.get("binary_path", "haddock3")

        # Check the properties
        self.check_properties(properties)

    @launchlogger
    def launch(self) -> int:
        """Execute the :class:`Topology <biobb_haddock.haddock.topology>` object."""
        # tmp_files = []

        # Setup Biobb
        if self.check_restart():
            return 0
        self.stage_files()

        workflow_dict = {
            "run_dir": fu.create_unique_dir(),
            "molecules": [self.stage_io_dict["in"]["mol1_input_pdb_path"]],
            "haddock_step_name": self.haddock_step_name,
        }
        workflow_dict.update(self.global_cfg)

        if mol2_input_pdb_path := self.stage_io_dict["in"].get("mol2_input_pdb_path"):
            workflow_dict["molecules"].append(mol2_input_pdb_path)

        # Create data dir
        cfg_dir = fu.create_unique_dir()
        self.output_cfg_path = create_cfg(
            output_cfg_path=str(Path(cfg_dir).joinpath(self.output_cfg_path)),
            workflow_dict=workflow_dict,
            input_cfg_path=self.stage_io_dict["in"].get("haddock_config_path"),
            cfg_properties_dict=self.cfg,
        )

        if self.container_path:
            fu.log("Container execution enabled", self.out_log)

            shutil.copy2(self.output_cfg_path, self.stage_io_dict.get("unique_dir", ""))
            self.output_cfg_path = str(
                Path(self.container_volume_path).joinpath(
                    Path(self.output_cfg_path).name
                )
            )

        self.cmd = [self.binary_path, self.output_cfg_path]

        # Run Biobb block
        self.run_biobb()

        # Copy files to host
        # self.copy_to_host()

        # Copy output
        haddock_output_path = Path(
            str(workflow_dict["run_dir"]), "0_" + self.haddock_step_name
        )
        mol1_name = str(Path(self.io_dict["in"]["mol1_input_pdb_path"]).stem)
        mol1_output_file_list = list(
            haddock_output_path.glob(mol1_name + r"*_haddock.pdb*")
        )
        fu.zip_list(
            self.io_dict["out"]["mol1_output_top_zip_path"],
            mol1_output_file_list,
            self.out_log,
        )

        if self.io_dict["out"].get("mol1_output_top_zip_path"):
            mol2_name = str(Path(self.io_dict["in"]["mol2_input_pdb_path"]).stem)
            mol2_output_file_list = list(
                haddock_output_path.glob(mol2_name + r"*_haddock.pdb*")
            )
            fu.zip_list(
                self.io_dict["out"]["mol2_output_top_zip_path"],
                mol2_output_file_list,
                self.out_log,
            )

        # Create zip output
        if self.io_dict["out"].get("output_haddock_wf_data_zip"):
            fu.log(
                f"Zipping {workflow_dict['run_dir']} to {str(Path(self.io_dict['out']['output_haddock_wf_data_zip']).with_suffix(''))} ",
                self.out_log,
                self.global_log,
            )
            shutil.make_archive(
                str(
                    Path(self.io_dict["out"]["output_haddock_wf_data_zip"]).with_suffix(
                        ""
                    )
                ),
                "zip",
                str(workflow_dict["run_dir"]),
            )

        # Remove temporary files
        self.tmp_files.extend([workflow_dict['run_dir'],
                               cfg_dir,
                               self.stage_io_dict.get("unique_dir")
                               ])
        self.remove_tmp_files()

        return self.return_code


def topology(
    mol1_input_pdb_path: str,
    mol1_output_top_zip_path: str,
    mol2_input_pdb_path: Optional[str] = None,
    mol2_output_top_zip_path: Optional[str] = None,
    output_haddock_wf_data_zip: Optional[str] = None,
    haddock_config_path: Optional[str] = None,
    properties: Optional[dict] = None,
    **kwargs,
) -> int:
    """Create :class:`Topology <biobb_haddock.haddock.topology>` class and
    execute the :meth:`launch() <biobb_haddock.haddock.topology.launch>` method."""

    return Topology(
        mol1_input_pdb_path=mol1_input_pdb_path,
        mol1_output_top_zip_path=mol1_output_top_zip_path,
        mol2_output_top_zip_path=mol2_output_top_zip_path,
        mol2_input_pdb_path=mol2_input_pdb_path,
        output_haddock_wf_data_zip=output_haddock_wf_data_zip,
        haddock_config_path=haddock_config_path,
        properties=properties,
        **kwargs,
    ).launch()


def main():
    parser = argparse.ArgumentParser(
        description="Wrapper of the haddock haddock module.",
        formatter_class=lambda prog: argparse.RawTextHelpFormatter(prog, width=99999),
    )
    parser.add_argument(
        "-c",
        "--config",
        required=False,
        help="This file can be a YAML file, JSON file or JSON string",
    )

    # Specific args of each building block
    required_args = parser.add_argument_group("required arguments")
    required_args.add_argument("--mol1_input_pdb_path", required=True)
    required_args.add_argument("--mol1_output_top_zip_path", required=True)
    parser.add_argument("--mol2_input_pdb_path", required=False)
    parser.add_argument("--output_haddock_wf_data_zip", required=False)
    parser.add_argument("--mol2_output_top_zip_path", required=False)
    parser.add_argument("--haddock_config_path", required=False)

    args = parser.parse_args()
    config = args.config if args.config else None
    properties = settings.ConfReader(config=config).get_prop_dic()

    # Specific call of each building block
    topology(
        mol1_input_pdb_path=args.mol1_input_pdb_path,
        mol1_output_top_zip_path=args.mol1_output_top_zip_path,
        mol2_output_top_zip_path=args.mol2_output_top_zip_path,
        mol2_input_pdb_path=args.mol2_input_pdb_path,
        output_haddock_wf_data_zip=args.output_haddock_wf_data_zip,
        haddock_config_path=args.haddock_config_path,
        properties=properties,
    )


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

"""Module containing the haddock3 run class and the command line interface."""

import argparse
import zipfile
import shutil
from pathlib import Path
from typing import Optional
import os
from biobb_common.configuration import settings
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.tools import file_utils as fu
from biobb_common.tools.file_utils import launchlogger


class Haddock3Extend(BiobbObject):
    """
    | biobb_haddock Haddock3Extend
    | Wrapper class for the Haddock3 extend module.
    | The `Haddock3 extend <https://www.bonvinlab.org/haddock3/tutorials/continuing_runs.html>`_. module continues the HADDOCK3 execution for docking of an already started run.

    Args:
        input_haddock_wf_data_zip (str): Path to the input zipball containing all the current Haddock workflow data. File type: output. `Sample file <https://github.com/bioexcel/biobb_haddock/raw/master/biobb_haddock/test/reference/haddock/ref_topology.zip>`_. Accepted formats: zip (edam:format_3987).
        haddock_config_path (str): Haddock configuration CFG file path. File type: input. `Sample file <https://raw.githubusercontent.com/bioexcel/biobb_haddock/master/biobb_haddock/test/data/haddock/run.cfg>`_. Accepted formats: cfg (edam:format_1476).
        output_haddock_wf_data_zip (str): Path to the output zipball containing all the current Haddock workflow data. File type: output. `Sample file <https://github.com/bioexcel/biobb_haddock/raw/master/biobb_haddock/test/reference/haddock/ref_topology.zip>`_. Accepted formats: zip (edam:format_3987).
        properties (dict - Python dictionary object containing the tool parameters, not input/output files):
            * **binary_path** (*str*) - ("haddock") Path to the haddock haddock executable binary.
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.
            * **sandbox_path** (*str*) - ("./") [WF property] Parent path to the sandbox directory.
            * **container_path** (*str*) - (None)  Path to the binary executable of your container.
            * **container_image** (*str*) - (None) Container Image identifier.
            * **container_volume_path** (*str*) - ("/data") Path to an internal directory in the container.
            * **container_working_dir** (*str*) - (None) Path to the internal CWD in the container.
            * **container_user_id** (*str*) - (None) User number id to be mapped inside the container.
            * **container_shell_path** (*str*) - ("/bin/bash") Path to the binary executable of the container shell.


    Examples:
        This is a use example of how to use the building block from Python::

            from biobb_haddock.haddock.haddock3_extend import haddock3_extend
            haddock3_extend(input_haddock_wf_data_zip='/path/to/myworkflowdata.zip',
                            haddock_config_path='/path/to/myHaddockConfig.cfg',
                            output_haddock_wf_data_zip='/path/to/haddock_output.zip',
                            properties=prop)

    Info:
        * wrapped_software:
            * name: Haddock3
            * version: 2025.5
            * license: Apache-2.0
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl
    """

    def __init__(
        self,
        input_haddock_wf_data_zip: str,
        haddock_config_path: str,
        output_haddock_wf_data_zip: str,
        properties: Optional[dict] = None,
        **kwargs,
    ) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)

        # Input/Output files
        self.io_dict = {
            "in": {
                "input_haddock_wf_data_zip": input_haddock_wf_data_zip,
                "haddock_config_path": haddock_config_path,
            },
            "out": {
                "output_haddock_wf_data_zip": output_haddock_wf_data_zip,
            },
        }

        # Properties specific for BB
        self.binary_path = properties.get("binary_path", "haddock3")

        # Check the properties
        self.check_properties(properties)

    @launchlogger
    def launch(self) -> int:
        """Execute the :class:`Haddock3Extend <biobb_haddock.haddock.haddock3_extend>` object."""

        # Setup Biobb
        if self.check_restart():
            return 0
        self.stage_files()

        # Decompress input zip
        run_dir = fu.create_unique_dir(self.stage_io_dict["unique_dir"])
        with zipfile.ZipFile(self.stage_io_dict["in"]["input_haddock_wf_data_zip"], 'r') as zip_ref:
            zip_ref.extractall(run_dir)
        cwd = os.getcwd()
        # Move the unzip folder
        os.chdir(run_dir)

        if self.container_path:
            fu.log("Container execution enabled", self.out_log)

            shutil.copy2(self.output_cfg_path, self.stage_io_dict.get("unique_dir", ""))
            self.output_cfg_path = str(
                Path(self.container_volume_path).joinpath(
                    Path(self.output_cfg_path).name
                )
            )

        self.cmd = [self.binary_path, self.stage_io_dict["in"]["haddock_config_path"]]
        self.cmd.extend(["--extend-run", run_dir])

        # Run Biobb block
        self.run_biobb()
        # Move back to the stage directory
        os.chdir(cwd)
        # Create zip output
        fu.log(
            f"Zipping {run_dir} to {str(Path(self.io_dict['out']['output_haddock_wf_data_zip']).with_suffix(''))} ",
            self.out_log, self.global_log)
        shutil.make_archive(
            str(Path(self.io_dict["out"]["output_haddock_wf_data_zip"]).with_suffix("")),
            "zip",
            str(run_dir),
        )

        # Remove temporal files
        self.tmp_files.extend([self.stage_io_dict.get("unique_dir"), run_dir])
        self.remove_tmp_files()

        return self.return_code


def haddock3_extend(
    input_haddock_wf_data_zip: str,
    haddock_config_path: str,
    output_haddock_wf_data_zip: str,
    properties: Optional[dict] = None,
    **kwargs,
) -> int:
    """Create :class:`Haddock3Extend <biobb_haddock.haddock.haddock3_extend>` class and
    execute the :meth:`launch() <biobb_haddock.haddock.haddock3_extend.launch>` method."""

    return Haddock3Extend(
        input_haddock_wf_data_zip=input_haddock_wf_data_zip,
        haddock_config_path=haddock_config_path,
        output_haddock_wf_data_zip=output_haddock_wf_data_zip,
        properties=properties,
        **kwargs,
    ).launch()


haddock3_extend.__doc__ = Haddock3Extend.__doc__


def main():
    parser = argparse.ArgumentParser(
        description="Wrapper of the haddock3 HADDOCK3 module.",
        formatter_class=lambda prog: argparse.RawTextHelpFormatter(prog, width=99999),
    )
    parser.add_argument(
        "-c",
        "--config",
        required=False,
        help="This file can be a YAML file, JSON file or JSON string",
    )

    # Specific args of each building block
    required_args = parser.add_argument_group("required arguments")
    required_args.add_argument("--input_haddock_wf_data_zip", required=True)
    required_args.add_argument("--haddock_config_path", required=True)
    required_args.add_argument("--output_haddock_wf_data_zip", required=True)

    args = parser.parse_args()
    config = args.config if args.config else None
    properties = settings.ConfReader(config=config).get_prop_dic()

    # Specific call of each building block
    haddock3_extend(
        input_haddock_wf_data_zip=args.input_haddock_wf_data_zip,
        haddock_config_path=args.haddock_config_path,
        output_haddock_wf_data_zip=args.output_haddock_wf_data_zip,
        properties=properties,
    )


if __name__ == "__main__":
    main()
